using System;
using System.Collections.Generic;
using System.IO;
using System.Web.Mvc;
using AutoDbCreate.WebHost.Models;
using AutoDbCreate.WebHost.Services;
using Castle.ActiveRecord;

namespace AutoDbCreate.WebHost.Controllers
{
    public class AdminController : Controller
    {
        private readonly ILocalDbManager _localDbManager;

        public AdminController(ILocalDbManager localDbManager)
        {
            _localDbManager = localDbManager;
        }

        public ActionResult Index()
        {
            return View();
        }

        public ActionResult UpdateSchema()
        {
            IList<Exception> exceptions = new List<Exception>();

            var dataDirectory = Convert.ToString(AppDomain.CurrentDomain.GetData("DataDirectory"));
            var mdfPath = Path.Combine(dataDirectory, "admin.mdf");

            // Create the schema's and capture errors during schema generation
            try { _localDbManager.CreateSchema(mdfPath, ARSchema.CUSTOMER); }
            catch (Exception e) { exceptions.Add(e); }
            try { _localDbManager.CreateSchema(mdfPath, ARSchema.LOGISTIC); }
            catch (Exception e) { exceptions.Add(e); }

            ActiveRecordStarter.UpdateSchema();

            ViewData["errors"] = exceptions;
            return RedirectToAction("Index"); 
        }

        public ActionResult GenerateScripts()
        {
            var dataDirectory = Convert.ToString(AppDomain.CurrentDomain.GetData("DataDirectory"));
            var mdfPath = Path.Combine(dataDirectory, "admin.mdf");

            var pathToSqlFile = Path.Combine(dataDirectory, "AdminSqlSchema.sql");
            _localDbManager.GenerateCreationScripts(mdfPath, pathToSqlFile);
            var fileContents = System.IO.File.ReadAllText(pathToSqlFile);
            ViewData["adminSchema"] = fileContents;
            return View();
        }

    }
}