using System;
using System.Data.SqlClient;
using Castle.ActiveRecord;

namespace AutoDbCreate.WebHost.Services
{
    /// <summary>
    /// Contract for a local db manager that creates local db's. 
    /// </summary>
    public interface ILocalDbManager : IService
    {
        /// <summary>
        /// Creates a db at <paramref name="pathToFile"/>
        /// </summary>
        /// <param name="pathToFile">The path where the db should be created.</param>
        void CreateDatabase(string pathToFile);

        /// <summary>
        /// Creates the <paramref name="schema"/> on the database located 
        /// at this <paramref name="path"/>
        /// </summary>
        /// <param name="path"></param>
        /// <param name="schema"></param>
        void CreateSchema(string path, string schema);

        void GenerateCreationScripts(string path, string pathToDump);
    }

    public class LocalDbManager : ILocalDbManager
    {
        public void CreateDatabase(string filename)
        {
            string databaseName = System.IO.Path.GetFileNameWithoutExtension(filename);
            using (var connection = new SqlConnection(
                @"Data Source=.\SQLEXPRESS;Initial Catalog=tempdb;Integrated Security=true;User Instance=True;"))
            {
                connection.Open();
                using (var command = connection.CreateCommand())
                {
                    command.CommandText =
                        "CREATE DATABASE " + databaseName +
                        " ON PRIMARY (NAME=" + databaseName +
                        ", FILENAME='" + filename + "')";
                    command.ExecuteNonQuery();

                    command.CommandText =
                        "EXEC sp_detach_db '" + databaseName + "', 'true'";
                    command.ExecuteNonQuery();
                }
            }
        }

        public void CreateSchema(string path, string schema)
        {
            var connectionString =
                String.Format(
                    @"Data Source=.\SQLEXPRESS;AttachDbFileName={0};Integrated Security=True;User Instance=True;", path);
            using (var connection = new SqlConnection(connectionString))
            {
                connection.Open();
                using (var command = connection.CreateCommand())
                {
                    command.CommandText = String.Format("CREATE SCHEMA [{0}]", schema);
                    command.ExecuteNonQuery();
                }
            }
        }

        public void GenerateCreationScripts(string path, string pathToDump)
        {
            ActiveRecordStarter.GenerateCreationScripts(pathToDump);
        }
    }
}